<?php declare(strict_types=1);

namespace App\Services\Translate\Completed\Project;

use App\Dto\Service\ProjectTranslationServiceTextHash\Hashes;
use App\Exceptions\Services\Translate\CompletedException;
use App\Jobs\Translate\ProcessTranslationText;
use App\Models\Project;
use App\Models\ProjectTranslation;
use App\Repositories\ProjectRepository;
use App\Repositories\ProjectTranslationRepository;
use App\Services\ProjectTranslationServiceTextHash\CompletionChecker;
use Illuminate\Support\Facades\DB;
use KorElf\TranslateLaravel\Contracts\TranslationCompletedListener;

final readonly class TranslationTextService implements TranslationCompletedListener
{
    public function __construct(
        private ProjectRepository $projectRepository,
        private CompletionChecker $completionChecker,
        private ProjectTranslationRepository $projectTranslationRepository,
    ) { }

    /**
     * @throws CompletedException
     */
    public function onTranslationCompleted(array $translatedText, array $data = []): void
    {
        if (
            !isset($data['projectId'])
            || !isset($data['languageId'])
            || !isset($data['hashes'])
        ) {
            throw new CompletedException('Required data is missing: projectId, languageId, or hashes.');
        }

        $project = $this->projectRepository->getProjectById((int) $data['projectId']);
        if ($project === null) {
            return;
        }

        $translations = $this->projectTranslationRepository->getProjectTranslationByCodes($project->id, $data['languageId'], \array_keys($translatedText))->all();

        $translateTextCode = DB::transaction(function () use ($data, $translatedText, $translations) {
            $translateTextCode = [];
            $hashes = $this->completionChecker->execute(
                $this->getHashes($data['hashes']),
            );

            foreach ($translatedText as $translatedTextKey => $translatedTextValue) {
                if ($hashes->isStatusWaiting($translatedTextKey) !== true) {
                    continue;
                }
                $translateTextCode[] = $translatedTextKey;
                $translation = $translations->firstWhere('code', $translatedTextKey);
                if ($translation !== null) {
                    $translation->update([
                        'text' => $translatedTextValue,
                    ]);
                    continue;
                }

                ProjectTranslation::create([
                    'project_id'  => (int) $data['projectId'],
                    'language_id' => (int) $data['languageId'],
                    'code'        => $translatedTextKey,
                    'text'        => $translatedTextValue,
                ]);
            }

            return $translateTextCode;
        });

        if (\config('translation_service.enable', false) && !empty($translateTextCode)) {
            $this->translateContent($project, $translateTextCode, $data);
        }
    }

    private function translateContent(Project $project, array $translateTextCode, array $data): void
    {
        $translateExceptLanguages = $data['exceptLanguages'] ?? [];
        $translateExceptLanguages[] = $data['languageId'];
        ProcessTranslationText::dispatch($project->id, $data['languageId'], $translateTextCode, $translateExceptLanguages);
    }

    private function getHashes(array $hashes): Hashes
    {
        $hashesDto = new Hashes();
        foreach ($hashes as $code => $hash) {
            $hashesDto->add((int) $hash['hashId'], $code, $hash['hash']);
        }
        return $hashesDto;
    }
}
